module Nfe
  module Models
    module Taxes
      class IPI
        include ActiveModel::Model
        include Helpers::XmlModel::NfeModel

        attr_xml :clEnq
        attr_xml :cnpjProd, xml_alias: "CNPJProd"
        attr_xml :cSelo
        attr_xml :qSelo

        attr_accessor :cst, :vBC, :pIPI, :qUnid, :vUnid, :vIPI

        validates :clEnq,
          length: { is: 5, allow_blank: true }

        validates :cnpjProd,
          cnpj: true,
          length: { is: 14, allow_blank: true }

        validates :cSelo,
          length: { maximum: 60, allow_blank: true }

        validates :qSelo,
          length: { maximum: 12, allow_blank: true }

        validates :cst,
          presence: true,
          inclusion: { in: ["00", "01", "02", "03", "04", "49", "50", "51", "52",
            "53", "54", "55", "99"]
          }

        def cEnq
          "999"
        end

        def present?
          check_present || @cst.present? || @vBC.present? || @pIPI.present? ||
            @qUnid.present? || @vUnid.present? || @vIPI.present?
        end

        def parse!(xml)
          doc = Nokogiri::XML(xml)

          parse_elements doc
          reset_values
          parse_when_IPITrib doc
          parse_when_IPINT doc
        end

        def to_xml
          xml = Builder::XmlMarkup.new

          add_elements xml, :clEnq, :cnpjProd, :cSelo, :qSelo

          xml.cEnq cEnq

          to_xml_when_IPITrib xml
          to_xml_when_IPINT xml

          xml.target!
        end

        private
        def IPITrib?
          ["00", "49", "50", "99"].include? @cst
        end

        def IPINT?
          ["01", "02", "03", "04", "05", "51", "52", "53", "54", "55"].include? @cst
        end

        def reset_values
          @cst = nil
          @vBC = nil
          @pIPI = nil
          @qUnid = nil
          @vUnid = nil
          @vIPI = nil
        end

        def parse_when_IPITrib(doc)
          if doc.css("IPITrib").present?
            ipi = IPITrib.new
            ipi.parse!(doc.css("IPITrib").to_xml)

            @cst = ipi.cst
            @vBC = ipi.vBC
            @pIPI = ipi.pIPI
            @qUnid = ipi.qUnid
            @vUnid = ipi.vUnid
            @vIPI = ipi.vIPI
          end
        end

        def parse_when_IPINT(doc)
          if doc.css("IPINT").present?
            ipi = IPINT.new
            ipi.parse!(doc.css("IPINT").to_xml)

            @cst = ipi.cst
          end
        end

        def to_xml_when_IPITrib(xml)
          if IPITrib?
            ipi = IPITrib.new

            ipi.cst = @cst
            ipi.vBC = @vBC
            ipi.pIPI = @pIPI
            ipi.qUnid = @qUnid
            ipi.vUnid = @vUnid
            ipi.vIPI = @vIPI

            xml.IPITrib { |node| node << ipi.to_xml }
          end
        end

        def to_xml_when_IPINT(xml)
          if IPINT?
            ipi = IPINT.new

            ipi.cst = @cst

            xml.IPINT { |node| node << ipi.to_xml }
          end
        end
      end
    end
  end
end
