module Nfe
  module Models
    module Taxes
      class PISCOFINS
        include ActiveModel::Model

        attr_accessor :cst, :vBC, :tax, :value, :qBCProd, :vAliqProd,
          :qBCProd

        validates :cst,
          presence: true,
          inclusion: { in: ["01", "02", "03", "04", "06", "07", "08", "09", "49",
            "50", "51", "52", "53", "54", "55", "56", "60", "61", "62", "63", "64",
            "65", "66", "67", "70", "71", "72", "73", "74", "75", "98", "99"]
          }

        def initialize(type)
          if type == "cofins"
            @type = "COFINS"
          else
            @type = "PIS"
          end
        end

        def present?
          @cst.present? || @vBC.present? || @tax.present? || @qBCProd.present? ||
          @vAliqProd.present? || @qBCProd.present?
        end

        def parse!(xml)
          doc = Nokogiri::XML(xml)

          reset_values
          parse_when_aliq(doc)
          parse_when_qtde(doc)
          parse_when_nt(doc)
          parse_when_outr(doc)
        end

        def to_xml
          xml = Builder::XmlMarkup.new

          to_xml_when_aliq(xml)
          to_xml_when_qtde(xml)
          to_xml_when_nt(xml)
          to_xml_when_outr(xml)

          xml.target!
        end

        private
        def aliq?
          PISCOFINSAliq.csts.include? @cst
        end

        def qtde?
          "03" == @cst
        end

        def nt?
          PISCOFINSNT.csts.include? @cst
        end

        def outr?
          PISCOFINSOutr.csts.include? @cst
        end

        def reset_values
          @cst = nil
          @vBC = nil
          @tax = nil
          @value = nil
          @qBCProd = nil
          @vAliqProd = nil
          @qBCProd = nil
        end

        def parse_when_aliq(doc)
          if doc.css("#{@type}Aliq").present?
            aliq = PISCOFINSAliq.new("p#{@type}", "v#{@type}")
            aliq.parse!(doc.css("#{@type}Aliq").to_xml)

            @cst   = aliq.cst
            @vBC   = aliq.vBC
            @tax   = aliq.tax
            @value = aliq.value
          end
        end

        def parse_when_qtde(doc)
          if doc.css("#{@type}Qtde").present?
            qtde = PISCOFINSQtde.new("v#{@type}")
            qtde.parse!(doc.css("#{@type}Qtde").to_xml)

            @cst       = qtde.cst
            @qBCProd   = qtde.qBCProd
            @vAliqProd = qtde.vAliqProd
            @value   = qtde.value
          end
        end

        def parse_when_nt(doc)
          if doc.css("#{@type}NT").present?
            nt = PISCOFINSNT.new
            nt.parse!(doc.css("#{@type}NT").to_xml)

            @cst = nt.cst
          end
        end

        def parse_when_outr(doc)
          if doc.css("#{@type}Outr").present?
            outr = PISCOFINSOutr.new("p#{@type}", "v#{@type}")
            outr.parse!(doc.css("#{@type}Outr").to_xml)

            @cst       = outr.cst
            @vBC       = outr.vBC
            @tax       = outr.tax
            @qBCProd   = outr.qBCProd
            @vAliqProd = outr.vAliqProd
            @value   = outr.value
          end
        end

        def to_xml_when_aliq(xml)
          if aliq?
            aliq = PISCOFINSAliq.new("p#{@type}", "v#{@type}")
            aliq.cst = @cst
            aliq.vBC = @vBC
            aliq.tax = @tax
            aliq.value = @value

            xml.tag!("#{@type}Aliq") { |node| node << aliq.to_xml }
          end
        end

        def to_xml_when_qtde(xml)
          if qtde?
            qtde = PISCOFINSQtde.new("v#{@type}")
            qtde.qBCProd   = @qBCProd
            qtde.vAliqProd = @vAliqProd
            qtde.value   = @value

            xml.tag!("#{@type}Qtde") { |node| node << qtde.to_xml }
          end
        end

        def to_xml_when_nt(xml)
          if nt?
            nt = PISCOFINSNT.new
            nt.cst     = @cst

            xml.tag!("#{@type}NT") { |node| node << nt.to_xml }
          end
        end

        def to_xml_when_outr(xml)
          if outr?
            outr = PISCOFINSOutr.new("p#{@type}", "v#{@type}")
            outr.cst       = @cst
            outr.vBC       = @vBC
            outr.tax       = @tax
            outr.qBCProd   = @qBCProd
            outr.vAliqProd = @vAliqProd
            outr.value   = @value

            xml.tag!("#{@type}Outr") { |node| node << outr.to_xml }
          end
        end
      end
    end
  end
end
