module Nfe
  module Operations
    class Base
      include ActiveModel::Model

      attr_reader :errors, :result

      def initialize(certificate, private_key, use_schema: true)
        @errors = ActiveModel::Errors.new(self)
        @certificate = certificate
        @private_key = private_key
        @use_schema = use_schema
        validate_certificate
      end

      def execute
        if errors.empty?
          before_execute
          call_webservice if valid?
        end

        errors.empty?
      end

      protected
      def before_execute
      end

      def valid?
        validate_with_validators
        validate_with_schema
        errors.empty?
      end

      private
      def call_webservice
        sender = create_webservice_client
        if sender.execute(@object)
          create_result(sender.result)
        elsif
          sender.errors.full_messages.each do |error|
            errors.add(:base, I18n.t("webservice.comunication", error: error))
          end
        end
      end

      def validate_with_validators
        errors.add(:base, I18n.t("xml.invalid")) unless @object.valid?
      end

      def validate_with_schema
        return if errors.any?

        if @use_schema
          validator = Helpers::SchemaValidator.new(schema, @object.to_xml)
          validator.valid?

          validator.errors.full_messages.each do |error|
            errors.add(:base, error)
          end
        end
      end

      def validate_certificate
        if @certificate.nil? || !(@certificate.instance_of? OpenSSL::X509::Certificate)
          errors.add(:base, I18n.t("webservice.invalid_certificate"))
        end

        if @private_key.nil? || !(@private_key.instance_of? OpenSSL::PKey::RSA)
          errors.add(:base, I18n.t("webservice.invalid_private_key"))
        end

        errors.empty?
      end

      def schema
        File.join(File.expand_path("../../schemas", __FILE__), schema_name)
      end

      def create_webservice_client
        Webservices::Sender.new(@certificate, @private_key, ws_info)
      end

      def create_result(response)
        @result = handle_result(parsed_response(response))
      rescue
        errors.add(:base, I18n.t("xml.unexpected_format",
          xml: response.to_xml))
      end

      def parsed_response(response)
        response.xpath(xpath, ns: Nfe::Models::NFe.namespace).to_xml
      end

      def sign(object, id)
        signer = Helpers::Signer.new(@certificate, @private_key)
        signer.execute(object, id)
      end
    end
  end
end
